sampler w; // vector field
sampler p; // pressure
uniform float halfrdx; // 0.5 / gridscale
uniform float2 invDim;
uniform float2 dim;


float texBilerpR(sampler tex, float2 s) {
  float4 st;
  st.xy = floor(dim*(s+invDim) - 0.5) + 0.5;
  st.zw = st.xy + 1;
  
  float2 t = invDim*(dim*(s+0.375f*invDim) - st.xy); //interpolating factors 
    
  float tex11 = tex2D(tex, invDim*st.xy).r;
  float tex21 = tex2D(tex, invDim*st.zy).r;
  float tex12 = tex2D(tex, invDim*st.xw).r;
  float tex22 = tex2D(tex, invDim*st.zw).r;

  // bilinear interpolation
  return lerp(lerp(tex11, tex21, t.x), lerp(tex12, tex22, t.x), t.y);
}

float2 texBilerpRG(sampler tex, float2 s) {
  float4 st;
  st.xy = floor(dim*(s+invDim) - 0.5) + 0.5;
  st.zw = st.xy + 1;
  
  float2 t = invDim*(dim*(s+0.375f*invDim) - st.xy); //interpolating factors 
    
  float2 tex11 = tex2D(tex, invDim*st.xy).rg;
  float2 tex21 = tex2D(tex, invDim*st.zy).rg;
  float2 tex12 = tex2D(tex, invDim*st.xw).rg;
  float2 tex22 = tex2D(tex, invDim*st.zw).rg;

  // bilinear interpolation
  return lerp(lerp(tex11, tex21, t.x), lerp(tex12, tex22, t.x), t.y);
}

/*void getFluidGradient(in float4 coords : TEXCOORD0, // grid coordinates
				out float2 uNew : COLOR0) // velocity
			{
	float pL = texBilerpR(p, coords - float2(invDim.x, 0));
	float pR = texBilerpR(p, coords + float2(invDim.x, 0));
	float pB = texBilerpR(p, coords - float2(0, invDim.y));
	float pT = texBilerpR(p, coords + float2(0, invDim.y));
	uNew = texBilerpRG(w, coords);
	//uNew.rg -= halfrdx * float2(pR - pL, pT - pB);
	uNew.rg -= invDim*float2(pR - pL, pT - pB);
	//uNew = float4(coords.xy, 1, 1);
}*/

void getFluidGradient(in float4 coords : TEXCOORD0, // grid coordinates
				out float2 uNew : COLOR0) // velocity
			{
	float pL = tex2D(p, coords - float2(invDim.x, 0)).r;
	float pR = tex2D(p, coords + float2(invDim.x, 0)).r;
	float pB = tex2D(p, coords - float2(0, invDim.y)).r;
	float pT = tex2D(p, coords + float2(0, invDim.y)).r;
	uNew = tex2D(w, coords).rg;
	//uNew.rg -= halfrdx * float2(pR - pL, pT - pB);
	//uNew.rg -= invDim*float2(pR - pL, pT - pB);
	uNew.rg -= 0.5f*float2(pR - pL, pT - pB);
	//uNew = float4(coords.xy, 1, 1);
}

pixelfragment fluidGradientPS = compile_fragment ps_3_0 getFluidGradient();