sampler x;
sampler b;

uniform float2 invDim;
uniform float2 dim;

uniform float alpha;
uniform float rBeta; // reciprocal beta

float2 texBilerpRG(sampler tex, float2 s) {
  float4 st;
  st.xy = floor(dim*(s+invDim) - 0.5) + 0.5;
  st.zw = st.xy + 1;
  
  float2 t = invDim*(dim*(s+0.375f*invDim) - st.xy); //interpolating factors 
    
  float2 tex11 = tex2D(tex, invDim*st.xy).rg;
  float2 tex21 = tex2D(tex, invDim*st.zy).rg;
  float2 tex12 = tex2D(tex, invDim*st.xw).rg;
  float2 tex22 = tex2D(tex, invDim*st.zw).rg;

  // bilinear interpolation
  return lerp(lerp(tex11, tex21, t.x), lerp(tex12, tex22, t.x), t.y);
}

/*void getFluidJacobiPS(float2 coords : TEXCOORD0, // grid coordinates
					out float2 xNew : COLOR) // b vector (Ax = b)
					{
	// left, right, bottom, and top x samples
	float2 xL = texBilerpRG(x, coords - float2(invDim.x, 0));
	float2 xR = texBilerpRG(x, coords + float2(invDim.x, 0));
	float2 xB = texBilerpRG(x, coords - float2(0, invDim.y));
	float2 xT = texBilerpRG(x, coords + float2(0, invDim.y));
	// b sample, from center
	float2 bC = texBilerpRG(b, coords);
	// evaluate Jacobi iteration
	xNew = (xL + xR + xB + xT + alpha * bC) * rBeta;
}*/
void getFluidJacobiPS(float2 coords : TEXCOORD0, // grid coordinates
					out float2 xNew : COLOR) // b vector (Ax = b)
					{
	// left, right, bottom, and top x samples
	float2 xL = tex2D(x, coords - float2(invDim.x, 0)).rg;
	float2 xR = tex2D(x, coords + float2(invDim.x, 0)).rg;
	float2 xB = tex2D(x, coords - float2(0, invDim.y)).rg;
	float2 xT = tex2D(x, coords + float2(0, invDim.y)).rg;
	// b sample, from center
	float2 bC = tex2D(b, coords).rg;
	// evaluate Jacobi iteration
	xNew = (xL + xR + xB + xT + alpha * bC) * rBeta;
}

pixelfragment fluidJacobiPS = compile_fragment ps_3_0 getFluidJacobiPS();