float4 marbleColor;
float4 baseColor;
float refractionIndex;
float fresnelFactor;

samplerCUBE samplerCubeMap:s0;

void firstMarblePS(in float4 texCoord0In: TEXCOORD0,
			    in float3 normalIn : NORMAL, 
				in float4 texCoord5In: TEXCOORD5,
			    out float4 texCoord0Out: r_texCoord0,
				out float4 texCoord5Out: r_texCoord5,
			    out float3 normalWorld: r_normalWorld) {
	texCoord0Out = texCoord0In;
	texCoord5Out = texCoord5In;
	normalWorld = normalIn;
}

pixelfragment psMarbleFirst = compile_fragment ps_3_0 firstMarblePS();


void getMarbleColor(in float4 texCoord0: r_texCoord0,
			  in float3 normalWorld: r_normalWorld,
			  in float4 viewVector: r_texCoord5,
			  out float4 outColor: COLOR) {

	viewVector = normalize(viewVector);
	normalWorld = normalize(normalWorld);
	
	float3 reflectVec = reflect(viewVector.xyz, normalWorld.xyz);
	float3 refractVec = refract(viewVector.xyz, normalWorld.xyz, refractionIndex);

	float4 reflectColor = texCUBE( samplerCubeMap, reflectVec );
	float4 refractColor = texCUBE( samplerCubeMap, refractVec );

	refractColor = refractColor*marbleColor;

	float powF = pow( 1 - dot( normalWorld.xyz, -viewVector.xyz ), 3 );

	//float F = g_fFresnelFactor + ( 1 - g_fFresnelFactor ) * pow( 1 - dot( N, -V ), 5 );
	float F = fresnelFactor + ( 1 - fresnelFactor ) * powF;
	outColor =  baseColor*(F * reflectColor) + ( 1 - F ) * refractColor;
}

pixelfragment psMarbleSecond = compile_fragment ps_3_0 getMarbleColor();