sampler sourceTexture;
sampler velocityTexture;
sampler prevVelocityTexture;


int NumberOfPostProcessSamples;
float PixelBlurConst;

void psDoBlur(in float4 texCoord0: TEXCOORD0,
			  out float4 outColor: COLOR) {

	//texCoord1 = texCoord0;

	float2 pixelVelocity;
    
    // Get this pixel's current velocity and this pixel's last frame velocity
    // The velocity is stored in .r & .g channels
    /*float4 curFramePixelVelocity = float4(tex2D(velocityTexture, texCoord1.xy).xy,0,0);
    float4 lastFramePixelVelocity = float4(tex2D(prevVelocityTexture, texCoord1.xy).xy,0,0);*/
	float4 cVel = tex2D(velocityTexture, texCoord0.xy);
	float2 curFramePixelVelocity = cVel.xy;
    float2 lastFramePixelVelocity = tex2D(prevVelocityTexture, texCoord0.xy).xy;

	float bias = 0.0001f;
	float cDepth = cVel.z;

    // If this pixel's current velocity is zero, then use its last frame velocity
    // otherwise use its current velocity.  We don't want to add them because then 
    // you would get double the current velocity in the center.  
    // If you just use the current velocity, then it won't blur where the object 
    // was last frame because the current velocity at that point would be 0.  Instead 
    // you could do a filter to find if any neighbors are non-zero, but that requires a lot 
    // of texture lookups which are limited and also may not work if the object moved too 
    // far, but could be done multi-pass.
    float curVelocitySqMag = curFramePixelVelocity.r * curFramePixelVelocity.r +
                             curFramePixelVelocity.g * curFramePixelVelocity.g;
    float lastVelocitySqMag = lastFramePixelVelocity.r * lastFramePixelVelocity.r +
                              lastFramePixelVelocity.g * lastFramePixelVelocity.g;
                                   
    if( lastVelocitySqMag > curVelocitySqMag )
    {
        pixelVelocity.x =  lastFramePixelVelocity.r * PixelBlurConst;   
        pixelVelocity.y = -lastFramePixelVelocity.g * PixelBlurConst;
    }
    else
    {
        pixelVelocity.x =  curFramePixelVelocity.r * PixelBlurConst;   
        pixelVelocity.y = -curFramePixelVelocity.g * PixelBlurConst;    
    }

	//pixelVelocity = clamp(pixelVelocity, -0.01f, 0.01f);
    
    // For each sample, sum up each sample's color in "Blurred" and then divide
    // to average the color after all the samples are added.
    float3 Blurred = 0;
	float4 Current = 0;
	float blurDepth = 0;
	int counter=0;
    for(float i = 0; i < NumberOfPostProcessSamples; i++)
    {   
        // Sample texture in a new spot based on pixelVelocity vector 
        // and average it with the other samples        
        float2 lookup = pixelVelocity* i / NumberOfPostProcessSamples + texCoord0.xy;
        
        
		blurDepth = tex2D(velocityTexture, lookup).z;
        
        // Add it with the other samples
		if(cDepth-bias<blurDepth) {
			// Lookup the color at this new spot
			Current = tex2D(sourceTexture, lookup);
			Blurred += Current.rgb;
			counter++;
		}
    }
    
    // Return the average color of all the samples
	//outColor = float4(Blurred / NumberOfPostProcessSamples, 1.0f);
	outColor = float4(Blurred / counter, 1.0f);


    //outColor = float4(curFramePixelVelocity, 0, 1);//float4(Blurred / NumberOfPostProcessSamples, 1.0f);
	//outColor = float4(Current.rgb, 1.0f);
	//outColor = float4(pixelVelocity, 0, 1.0f);

    //outColor = tex2D(prevVelocityTexture, texCoord0.xy);
	//outColor = tex2D(velocityTexture, texCoord0.xy) + tex2D(prevVelocityTexture, texCoord0.xy)+tex2D(sourceTexture, texCoord0.xy);/* - tex2D(sourceTexture, texCoord0.xy)*/;
}

pixelfragment psDepthMotion = compile_fragment ps_3_0 psDoBlur();