sampler2D diffuseTex;
sampler2D normalTex;
sampler2D heightTex;

float4x4 worldMat;

float invFarMinusNear;

void getParallaxColor(in float4 texCoord0: r_texCoord0,
					  in float4 texCoord1: TEXCOORD1,
					  in float4 texCoord2: TEXCOORD2,
					  in float4 texCoord3: TEXCOORD3,
					   in float4 depth: TEXCOORD4,
					 in float3 viewTangent: TEXCOORD5,
					 in float4 view: TEXCOORD6,
					 out float4 outColor: COLOR0,
					 out float2 outDepth: COLOR1) {


	// Parallax zeugs
	float fHeightMapScale=0.02;
	/*float nMinSamples=100;
	float nMaxSamples=150;*/
	float nMinSamples=60;
	float nMaxSamples=80;
	float fParallaxLimit = length(viewTangent.xy) / viewTangent.z;
	fParallaxLimit *= fHeightMapScale;
	float2 vOffset = normalize( -viewTangent.xy );
	viewTangent=normalize(viewTangent);
	vOffset = vOffset * fParallaxLimit;

	int nNumSamples = (int)lerp( nMinSamples, nMaxSamples, dot( viewTangent, float3(0.0,0.0,1.0)));
	nNumSamples*=saturate(1.0-5.0*depth.x);

	float fStepSize = 1.0 / (float)nNumSamples;

	float2 dx, dy;
	dx = ddx( texCoord0 );
	dy = ddy( texCoord0 );

	float2 vOffsetStep = fStepSize * vOffset;
	float2 vCurrOffset = float2( 0, 0 );
	float2 vLastOffset = float2( 0, 0 );
	float2 vFinalOffset = float2( 0, 0 );

	float stepLen=length(vOffsetStep);
	float currLen=0.0;

	float4 vCurrSample;
	float4 vLastSample;

	float stepHeight = 1.0;
	int nCurrSample = 0;

	while ( nCurrSample < nNumSamples )
	{
	   //vCurrSample = tex2Dgrad( diffuseTex, texCoord0 + vCurrOffset, dx, dy );
		vCurrSample = tex2Dgrad( heightTex, texCoord0 + vCurrOffset, dx, dy );
	   if ( vCurrSample.r > stepHeight )
	   {
		  vFinalOffset = vLastOffset;

		 // vCurrSample = tex2Dgrad( diffuseTex, texCoord0 + vFinalOffset, dx, dy );
		  nCurrSample = nNumSamples + 1;
	   }
	   else
	   {

		  nCurrSample++;
		  stepHeight -= fStepSize;
		  vLastOffset = vCurrOffset;
		  vCurrOffset += vOffsetStep;
		  currLen+=stepLen;
		  vLastSample = vCurrSample;
	   }
	}

	// Normalmapping
	
	//float3 lvec1=normalize(float3(1.0,1.0,0.0));
	float3 lvec1=normalize(float3(0.0,1.0,1.0));
	float3 l1_diff=float3(1.0,1.0,1.0);
	float3 l1_amb=float3(0.05,0.05,0.05);
	float3 lightspec1=float3(1.0,1.0,1.0);
	float3 matspec=float3(0.1,0.1,0.1);


	float shininess=2.0;

	//float4 color=tex2D(diffuseTex,float2(texCoord0.xy)+vFinalOffset)*(0.5+tex2Dgrad(heightTex,float2(texCoord0.xy)+vFinalOffset,dx, dy).rgba);
	float4 color=tex2D(diffuseTex,float2(texCoord0.xy)+vFinalOffset)*(0.5+tex2Dgrad(heightTex,float2(texCoord0.xy)+vFinalOffset,dx, dy).r);
	//float4 color=(tex2D(diffuseTex,float2(texCoord0.xy)+vFinalOffset)+tex2Dgrad(heightTex,float2(texCoord0.xy)+vFinalOffset,dx, dy))*0.5;
	//float4 color=tex2D(diffuseTex,float2(texCoord0.xy)+vFinalOffset);
//*(0.5+vCurrSample.r);//
	/*float3x3 tbnMatrix=float3x3(texCoord1.xyz, texCoord2.xyz, texCoord3.xyz);

	float3 n=2.0*(tex2D(normalTex, float2(texCoord0.xy)+vFinalOffset).rgb-0.5);
	n.y=-n.y;
	n=normalize(mul(mul( n, tbnMatrix ),(float3x3) worldMat ));
	
	float3 halfV1=reflect(-lvec1.xyz, n);

	float4 diffuse1  = float4(color.xyz*l1_diff * max(0.0, dot(n, lvec1.xyz)),1.0);

	float4 amb1=float4(color.xyz*l1_amb,1.0);

	float specularPower=pow(max(0.0, -dot(halfV1, view.xyz)), shininess);
	float4 specular1=float4(lightspec1*specularPower*matspec,1.0);*/



	//outColor= float4((amb1+diffuse1+specular1).xyz,1);
	outColor= float4(color.xyz,1);
	outColor.w=1.0;

	//outColor=tex2D(heightTex, float2(texCoord0.xy));

	depth.x+=currLen;//*invFarMinusNear;

	outDepth = depth.x;
	//outColor=float4(amb1.xyz,1.0);//float4(1, 0, 0, 1);
	//outDepth = (amb1+diffuse1+specular1).x;
}

pixelfragment psParallaxColor = compile_fragment ps_3_0 getParallaxColor();