sampler2D heightTex;

//float4x4 worldMat;

float invFarMinusNear;

void getParallaxDepth(in float4 texCoord0: TEXCOORD0,
					  in float4 depth: TEXCOORD4,
					 in float3 viewTangent: TEXCOORD5,
					 out float4 outColor: COLOR) {


	// Parallax zeugs
	float fHeightMapScale=0.005;
	float nMinSamples=80;
	float nMaxSamples=200;
	float fParallaxLimit = length(viewTangent.xy) / viewTangent.z;
	fParallaxLimit *= fHeightMapScale;
	float2 vOffset = normalize( -viewTangent.xy );
	viewTangent=normalize(viewTangent);
	vOffset = vOffset * fParallaxLimit;

	int nNumSamples = (int)lerp( nMinSamples, nMaxSamples, dot( viewTangent, float3(0.0,0.0,1.0) ) );

	float fStepSize = 1.0 / (float)nNumSamples;

	float2 dx, dy;
	dx = ddx( texCoord0 );
	dy = ddy( texCoord0 );

	float2 vOffsetStep = fStepSize * vOffset;
	float2 vCurrOffset = float2( 0, 0 );
	float2 vLastOffset = float2( 0, 0 );
	float2 vFinalOffset = float2( 0, 0 );

	float stepLen=length(vOffsetStep);
	float currLen=0.0;

	float4 vCurrSample;
	float4 vLastSample;

	float stepHeight = 1.0;
	int nCurrSample = 0;

	while ( nCurrSample < nNumSamples )
	{
		vCurrSample = tex2Dgrad( heightTex, texCoord0 + vCurrOffset, dx, dy );
	   if ( vCurrSample.r > stepHeight )
	   {
		  float Ua = (vLastSample.r - (stepHeight+fStepSize))
					  / ( fStepSize + (vCurrSample.r - vLastSample.r));
		  vFinalOffset = vLastOffset + Ua * vOffsetStep;

		  nCurrSample = nNumSamples + 1;
	   }
	   else
	   {
		  nCurrSample++;
		  stepHeight -= fStepSize;
		  vLastOffset = vCurrOffset;
		  vCurrOffset += vOffsetStep;
		  currLen+=stepLen;
		  vLastSample = vCurrSample;
	   }
	}

	depth.x+=currLen*invFarMinusNear;;
	depth.x*=20.0;
	outColor=float4(depth.xxx,1.0);//depth;
	//outColor = tex2D(heightTex, texCoord0.xy);
	//outColor=float4(0.5f,0.5f,0.5f,1.0);
}

pixelfragment psParallaxDepth = compile_fragment ps_3_0 getParallaxDepth();