/*sampler lightDepthTex;

//**************** LEFT *****************

float findLeftBorder(in float2 left,
					 in float2 center,
					 in float threshold) {

	float2 dir = left - center;
	float fullLength = length(dir);

	dir*=0.5f;

	float2 currentPos;
	float bias = 0.0045f;
	float currentDepth;
	currentPos = center + dir;

	for(int i=0;i<4;i++) {
		currentDepth = tex2Dlod(lightDepthTex, float4(currentPos, 0, 0)).r + bias;
		dir*=0.5f;
		if(currentDepth<threshold) {
			currentPos += dir;
		} else {
			currentPos -= dir;
		}
	}

	return length(currentPos - center);
}

//**************** RIGHT *****************

float findRightBorder(in float2 center,
					 in float2 right,
					 in float threshold) {

	float2 dir = right - center;
	float fullLength = length(dir);

	dir*=0.5f;

	float2 currentPos;
	float bias = 0.0045f;
	float currentDepth;
	currentPos = center + dir;

	for(int i=0;i<4;i++) {
		currentDepth = tex2Dlod(lightDepthTex, float4(currentPos, 0, 0)).r + bias;
		dir*=0.5f;
		if(currentDepth<threshold) {
			currentPos -= dir;
		} else {
			currentPos += dir;
		}
	}

	return length(currentPos - right);
}

float calcScaleFactor(in float cDepth, in float texDepth) {
	return 30*max(cDepth - texDepth, 0);
}

float getScaleFactor(in bool rightInShadow, in bool centerInShadow, in bool leftInShadow,
					 in float cRightDepth, in float rightDepth,
					 in float cCenterDepth, in float centerDepth,
					 in float cLeftDepth, in float leftDepth) {

	float scaleFactor = 1.0f;
	if(rightInShadow && !centerInShadow && !leftInShadow) {
		scaleFactor = calcScaleFactor(cRightDepth, rightDepth);
	}
	if(centerInShadow) {
		scaleFactor = calcScaleFactor(cCenterDepth, centerDepth);
	}
	if(leftInShadow && !centerInShadow && !rightInShadow) {
		scaleFactor = calcScaleFactor(cLeftDepth, leftDepth);
	}
	return scaleFactor;
}

void getShadowSize(in float4 texCoord6, 
					in float4 texCoord7, 
					in out float shadowSize,
					in out float basis) {
	
		float2 centerUV = float2((texCoord7.xy/texCoord7.w)+0.5f);
		float2 leftUV = float2((texCoord6.xy/texCoord6.w)+0.5f);
		float2 rightUV = 2*centerUV - leftUV;
		//basis = length(rightUV - leftUV);
		float2 leftDir = leftUV - centerUV;
		
		
		float bias = 0.0008f;
		//float bias = 0.0004f;

		//Change leftUV and rightUV depending on depth difference
		float cCenterDepth = texCoord7.z;
		float cLeftDepth = texCoord6.z;
		float cRightDepth = 2*cCenterDepth - cLeftDepth;
		float centerDepth = tex2Dlod(lightDepthTex, float4(centerUV,0,0)).r + bias;
		float leftDepth = tex2Dlod(lightDepthTex, float4(leftUV,0,0)).r + bias;
		float rightDepth = tex2Dlod(lightDepthTex, float4(rightUV,0,0)).r + bias;	

		bool centerInShadow = centerDepth<cCenterDepth;
		bool leftInShadow = leftDepth<cLeftDepth;
		bool rightInShadow = rightDepth<cRightDepth;

		float scaleFactor = getScaleFactor(rightInShadow, centerInShadow, leftInShadow,
										   cRightDepth, rightDepth,
										   cCenterDepth, centerDepth,
										   cLeftDepth, leftDepth);
		

		leftUV = centerUV + leftDir*scaleFactor;
		rightUV = 2*centerUV - leftUV;

		leftDepth = tex2Dlod(lightDepthTex, float4(leftUV,0,0)).r + bias;
		rightDepth = tex2Dlod(lightDepthTex, float4(rightUV,0,0)).r + bias;	

		leftInShadow = leftDepth<cLeftDepth;
		rightInShadow = rightDepth<cRightDepth;
		
		basis = length(rightUV - leftUV);
		float halfBasis = basis*0.5f;
		

		//Left Shadow Edge
		if(rightInShadow && !centerInShadow && !leftInShadow) {
			shadowSize = findLeftBorder(centerUV, rightUV, cCenterDepth);
		}
		else if(rightInShadow && centerInShadow && !leftInShadow) {
			shadowSize = halfBasis + findLeftBorder(leftUV, centerUV, cCenterDepth);
		}

		//Right Shadow Edge
		else if(leftInShadow && !centerInShadow && !rightInShadow) {
			shadowSize = findRightBorder(centerUV, leftUV, cCenterDepth);
		}
		else if(leftInShadow && centerInShadow && !rightInShadow) {
			shadowSize = halfBasis + findRightBorder(rightUV, centerUV, cCenterDepth);
		}

		//Center shadowed, but not left and right side
		else if(!leftInShadow && centerInShadow && !rightInShadow) {
			shadowSize = findLeftBorder(rightUV, centerUV, cCenterDepth) + findRightBorder(leftUV, centerUV, cCenterDepth);
		}

		//Sides shadowed, but not center
		else if(leftInShadow && !centerInShadow && rightInShadow) {
			shadowSize = findRightBorder(centerUV, leftUV, cCenterDepth) + findLeftBorder(centerUV, rightUV, cCenterDepth);
		}

		//Full Shadow
		else if(leftInShadow && centerInShadow && rightInShadow) {
			shadowSize = basis;
		}

		else {
			shadowSize = 0;
		}
}

void getStdLightPS(in float4 matColor: r_matColor,
				   in float4 texCoord0: r_texCoord0,
				   in float4 texCoord3: TEXCOORD3,
				   in float4 texCoord6: TEXCOORD6,
				   in float4 texCoord7: TEXCOORD7,
				   out float4 resultColor: r_resultColor) {
   
	if(texCoord7.w>1.0f) {
		/*float shadowSizeHor = 0;
		float shadowSizeVer = 0;
		float basisHor = 0;
		float basisVer = 0;
		getShadowSize(texCoord3, texCoord7, shadowSizeHor, basisHor);
		shadowFactor = 0.4f + 0.6f*((basisHor - shadowSizeHor)/basisHor);
		resultColor = matColor*shadowFactor;*/


		/*float shadowSizeHor = 0;
		float shadowSizeVer = 0;
		float basisHor = 0;
		float basisVer = 0;
		getShadowSize(texCoord6, texCoord7, shadowSizeHor, basisHor);
		getShadowSize(texCoord3, texCoord7, shadowSizeVer, basisVer);

		float a=min(shadowSizeHor/basisHor, shadowSizeVer/basisVer);
		float shadowSize=0.0;
		if(a<0.5) {
			shadowSize=max(shadowSizeHor,shadowSizeVer);
		} else {
			shadowSize=min(shadowSizeHor, shadowSizeVer);
		}

		float lightArea = (basisHor+basisVer)*0.5f;
		//float shadowSize = (shadowSizeHor+shadowSizeVer)*0.5f;
		float shadowFactor = 0.4f + 0.6f*((lightArea-shadowSize)/lightArea);
		resultColor = matColor*shadowFactor;
	} else {
		resultColor = matColor*0.4f;
	}
	//resultColor = tex2D(lightDepthTex, centerUV.xy).rrrr*1000;
}

pixelfragment stdLightPS = compile_fragment ps_3_0 getStdLightPS();*/




sampler lightDepthTex;

//**************** LEFT *****************

float findLeftBorder(in float2 left,
					 in float2 center,
					 in float threshold) {

	float2 dir = left - center;
	float fullLength = length(dir);

	dir*=0.5f;

	float2 currentPos;
	float bias = 0.0005f;
	float currentDepth;
	currentPos = center + dir;

	for(int i=0;i<4;i++) {
		currentDepth = tex2Dlod(lightDepthTex, float4(currentPos, 0, 0)).r + bias;
		dir*=0.5f;
		if(currentDepth<threshold) {
			currentPos += dir;
		} else {
			currentPos -= dir;
		}
	}

	return length(currentPos - center);
}

//**************** RIGHT *****************

float findRightBorder(in float2 center,
					 in float2 right,
					 in float threshold) {

	float2 dir = right - center;
	float fullLength = length(dir);

	dir*=0.5f;

	float2 currentPos;
	float bias = 0.0005f;
	float currentDepth;
	currentPos = center + dir;

	for(int i=0;i<4;i++) {
		currentDepth = tex2Dlod(lightDepthTex, float4(currentPos, 0, 0)).r + bias;
		dir*=0.5f;
		if(currentDepth<threshold) {
			currentPos -= dir;
		} else {
			currentPos += dir;
		}
	}

	return length(currentPos - right);
}

float calcScaleFactor(in float cDepth, in float texDepth) {
	return 30*max(cDepth - texDepth, 0);
}

float getScaleFactor(in bool rightInShadow, in bool centerInShadow, in bool leftInShadow,
					 in float cRightDepth, in float rightDepth,
					 in float cCenterDepth, in float centerDepth,
					 in float cLeftDepth, in float leftDepth) {

	float scaleFactor = 1.0f;
	if(rightInShadow && !centerInShadow && !leftInShadow) {
		scaleFactor = calcScaleFactor(cRightDepth, rightDepth);
	}
	if(centerInShadow) {
		scaleFactor = calcScaleFactor(cCenterDepth, centerDepth);
	}
	if(leftInShadow && !centerInShadow && !rightInShadow) {
		scaleFactor = calcScaleFactor(cLeftDepth, leftDepth);
	}
	return scaleFactor;
}

void getStdLightPS(in float4 matColor: r_matColor,
				   in float4 texCoord0: r_texCoord0,
				   in float4 texCoord6: TEXCOORD6,
				   in float4 texCoord7: TEXCOORD7,
				   out float4 resultColor: r_resultColor) {
   
	float shadowFactor = 1.0f;

	if(texCoord7.w>1.0f) {
		float2 centerUV = float2((texCoord7.xy/texCoord7.w)+0.5f);
		float2 leftUV = float2((texCoord6.xy/texCoord6.w)+0.5f);
		float2 leftDir = leftUV - centerUV;
		float2 rightUV = 2*centerUV - leftUV;
		
		float bias = 0.00025f;
		//float bias = 0.0004f;

		//Change leftUV and rightUV depending on depth difference
		float cCenterDepth = texCoord7.z;
		float cLeftDepth = texCoord6.z;
		float cRightDepth = 2*cCenterDepth - cLeftDepth;
		float centerDepth = tex2Dlod(lightDepthTex, float4(centerUV,0,0)).r + bias;
		float leftDepth = tex2Dlod(lightDepthTex, float4(leftUV,0,0)).r + bias;
		float rightDepth = tex2Dlod(lightDepthTex, float4(rightUV,0,0)).r + bias;	

		bool centerInShadow = centerDepth<cCenterDepth;
		bool leftInShadow = leftDepth<cLeftDepth;
		bool rightInShadow = rightDepth<cRightDepth;

		float scaleFactor = getScaleFactor(rightInShadow, centerInShadow, leftInShadow,
										   cRightDepth, rightDepth,
										   cCenterDepth, centerDepth,
										   cLeftDepth, leftDepth);
		

		leftUV = centerUV + leftDir*scaleFactor;
		rightUV = 2*centerUV - leftUV;

		leftDepth = tex2Dlod(lightDepthTex, float4(leftUV,0,0)).r + bias;
		rightDepth = tex2Dlod(lightDepthTex, float4(rightUV,0,0)).r + bias;	

		leftInShadow = leftDepth<cLeftDepth;
		rightInShadow = rightDepth<cRightDepth;
		
		float basis = length(rightUV - leftUV);
		float halfBasis = basis*0.5f;
		
		float shadowSize = 0;

		//Left Shadow Edge
		if(rightInShadow && !centerInShadow && !leftInShadow) {
			shadowSize = findLeftBorder(centerUV, rightUV, cCenterDepth);
		}
		else if(rightInShadow && centerInShadow && !leftInShadow) {
			shadowSize = halfBasis + findLeftBorder(leftUV, centerUV, cCenterDepth);
		}

		//Right Shadow Edge
		else if(leftInShadow && !centerInShadow && !rightInShadow) {
			shadowSize = findRightBorder(centerUV, leftUV, cCenterDepth);
		}
		else if(leftInShadow && centerInShadow && !rightInShadow) {
			shadowSize = halfBasis + findRightBorder(rightUV, centerUV, cCenterDepth);
		}

		//Center shadowed, but not left and right side
		else if(!leftInShadow && centerInShadow && !rightInShadow) {
			shadowSize = findLeftBorder(rightUV, centerUV, cCenterDepth) + findRightBorder(leftUV, centerUV, cCenterDepth);
		}

		//Sides shadowed, but not center
		else if(leftInShadow && !centerInShadow && rightInShadow) {
			shadowSize = findRightBorder(centerUV, leftUV, cCenterDepth) + findLeftBorder(centerUV, rightUV, cCenterDepth);
		}

		//Full Shadow
		else if(leftInShadow && centerInShadow && rightInShadow) {
			shadowSize = basis;
		}

		shadowFactor = 0.4f + 0.6f*((basis-shadowSize)/basis);
		resultColor = matColor*shadowFactor;
	} else {
		resultColor = matColor*0.4f;
	}
	//resultColor = tex2D(lightDepthTex, centerUV.xy).rrrr*1000;
}

pixelfragment stdLightPS = compile_fragment ps_3_0 getStdLightPS();