//------------------------------------
float4x4 worldMat;
float4x4 worldViewProjMat;
float4x4 prevWorldViewProjMat;
float invFarMinusNear;


//------------------------------------
//Do just simply copy the position and normal to shared register
void getFirstDepthBlur(in float4 pos: POSITION,
					   in float3 normal: NORMAL,
					   in float4 texCoord0: TEXCOORD0,
					   in float4 texCoord7: TEXCOORD7,
					   out float4 posVertex: r_posVertex,
					   out float3 normalOut: r_normal,
					   out float4 texCoord0Out: r_texCoord0,
					   out float4 texCoord7Out: r_texCoord7) 
{
	posVertex = pos;
	normalOut = normalize(normal);
	texCoord0Out = texCoord0;
	texCoord7Out = texCoord7;
}

vertexfragment firstDepthBlur = compile_fragment vs_3_0 getFirstDepthBlur();


//Perform projection of pos and normal
void getSecondDepthBlur(in float4 posVertex: r_posVertex,
				in float3 normal: r_normal,
				in float4 texCoord0: r_texCoord0,
				in float4 texCoord7: r_texCoord7,
				out float4 hpos: POSITION,
				out float3 normalWorld: NORMAL,
				out float4 texCoord0Out: TEXCOORD0,
				out float4 texCoord6Out: TEXCOORD6,
				out float4 texCoord7Out: TEXCOORD7) {

	
    float4 vPosProjSpaceCurrent; 
    float4 vPosProjSpaceLast; 
  
    normalWorld = normalize(mul(normal, (float3x3)worldMat)); // normal (world space)
    
    // Transform from object space to homogeneous projection space
    vPosProjSpaceCurrent = mul(posVertex, worldViewProjMat);
    vPosProjSpaceLast = mul(posVertex, prevWorldViewProjMat);
    
    // Output the vetrex position in projection space
    hpos = vPosProjSpaceCurrent;

    // Convert to non-homogeneous points [-1,1] by dividing by w 
	float depth = vPosProjSpaceCurrent.z*invFarMinusNear;
	float depthLinear = vPosProjSpaceCurrent.z;
    //vPosProjSpaceCurrent /= vPosProjSpaceCurrent.w;
    //vPosProjSpaceLast /= vPosProjSpaceLast.w;
	texCoord6Out = vPosProjSpaceCurrent;
	texCoord7Out = float4(vPosProjSpaceLast.xy, depth, vPosProjSpaceLast.w);
    
    // Vertex's velocity (in non-homogeneous projection space) is the position this frame minus 
    // its position last frame.  This information is stored in a texture coord.  The pixel shader 
    // will read the texture coordinate with a sampler and use it to output each pixel's velocity.
    float2 velocity = (vPosProjSpaceCurrent - vPosProjSpaceLast).xy;   
    
    // The velocity is now between (-2,2) so divide by 2 to get it to (-1,1)
    velocity /= 0.5f;
	//velocity = clamp(velocity, -1.0f, 1.0f);
	//velocity += 0.5f;
	//texCoord0Out = texCoord0;

    // Store the velocity in a texture coord
    //texCoord7Out = float4(velocity.x, velocity.y, depth, 1.0f);
 
	texCoord0Out = texCoord0;
}

vertexfragment secondDepthBlur = compile_fragment vs_3_0 getSecondDepthBlur();
 